<?php

define("GENERATE_REPORT_ERROR_NOSERVER", -1);

class ReportGenerator {
	
	static $URL = "cron_admin.pl?";
	
	private $SERVER;
	private $arguments = array();
	private $dataType = null;
	private $classifications = array();
	private $classificationValues = array();

	private $anonymousClassificationInternalIds = array();
		
	function __construct($server = null) {
		$this->SERVER = $server;
		$this->setArgument("command", "instant");	// default value
	}
	
	
	// Set variables
	function setArgument($name = null, $value = null) {
		if (is_null($name) || is_null($value)) return;
		$this->arguments[$name] = $value;
	}

	
	function setResolution($index) {
		switch ($index) {
			case 0: $value = "300"; break;
			case 1: $value = "900"; break;
			case 2: $value = "3600"; break;
			case 3: $value = "86400"; break;
			case 4: $value = "604800"; break;
			case 5: $value = "M"; break;
			case 6: $value = "Y"; break;
			default: return;
		}
		$this->setArgument("resolution", $value);
	}
	
	function setPresentationType($presentationType = null) {
		$this->setArgument("presentation_type", $presentationType);
	}
	
	function setFolder($folder = null) {
		$this->setArgument("folder", trim($folder));
	}
	
	function setTitle($title = null) {
		$this->setArgument("title", $title);
	}
	
	function setFromDate($date, $time = "00:00:00") {
		if (strlen($date) != 10) $date = date("Y-m-d", mktime(0, 0, 0, 1, 1, date("Y")-1));
		if (strlen($time) != 8) $time = "00:00:00";
		$this->setArgument("from", $date . " " . $time);
	}
	
	function setFrom($timestamp) {
		$this->setArgument("from", $timestamp);
	}

	function setTo($timestamp) {
		$this->setArgument("to", $timestamp);
	}
		
	function setToDate($date, $time = "00:00:00") {
		if (strlen($date) != 10) $date = date("Y-m-d", mktime(0, 0, 0, 1, 1, date("Y")-1));
		if (strlen($time) != 8) $time = "00:00:00";
		$this->setArgument("to", $date . " " . $time);
	}
	
	function setDataType($dataType) {
		$this->dataType = $dataType;
		
		$measurements = $this->dataType->getMeasurements();
		$accvars = "";
		foreach ($measurements as $measurement) {	
			if ($measurement->isSelected()) {
				if ($accvars != "") $accvars .= ",";
				$accvars .= $measurement->getId();
				Logger::Debug("ReportGenerator", "Adding measurement '" . $measurement->getValue() . "' to report");
			}
		}
		$this->setArgument("accvars", $accvars);
		$this->setArgument("acctype", $this->dataType->getUID());
	}
	
	function addClassification($classification = null, $selectedValues = array()) {
		if (is_null($classification)) return;
		$this->classifications[$classification->getID()] = $classification;
		if (is_array($selectedValues) && count($selectedValues) > 0) {
			$this->classificationValues[$classification->getID()] = $selectedValues;
		}
	}
		
	function setAnonymous($classificationInternalId) {
		$this->anonymousClassificationInternalIds[$classificationInternalId] = 1;
	}
	
	
	/**
	 * Check if the report can be generated
	 *
	 * @return boolean
	 */
	function isValid() {
		
		// Generation stuff		
		if (empty($this->arguments["title"])) {
			Logger::Debug("ReportGenerator::isValid()", "Report doesn’t have a title");
			return false; 
		}
		//if (empty($this->arguments["from"])) return false; 
		//if (empty($this->arguments["to"])) return false; 
		if (empty($this->arguments["folder"])) {
			Logger::Debug("ReportGenerator::isValid()", "Report doesn’t have a output folder");
			return false; 
		}
		if (empty($this->arguments["presentation_type"])) {
			Logger::Debug("ReportGenerator::isValid()", "Report doesn’t have a presentation type");
			return false; 
		}
		if (!is_numeric($this->arguments["acctype"])) {
			Logger::Debug("ReportGenerator::isValid()", "Report doesn’t have any selected datatype/acctype");
			return false; 	// Data Type
		}
		// Presentation variables
		if ($this->arguments["presentation_type"] == "chart") {

			// Chart variables
			if (!isset($this->arguments["accvars"]) || $this->arguments["accvars"] == "") {
				Logger::Debug("ReportGenerator::isValid()", "Chart report doesn’t have a measurement");
				return false; 	// Measures
			}
			
			if (!isset($this->arguments["pnum"]) || $this->arguments["pnum"] == "") {
				Logger::Debug("ReportGenerator::isValid()", "Chart report doesn’t have a primary classification");
				return false;	// 1. Classifications
			}
			
			if (!isset($this->arguments["snum"]) || $this->arguments["snum"] == "") {
				Logger::Debug("ReportGenerator::isValid()", "Chart report doesn’t have a secondary classification");
				return false;	// 2. Classifications
			}
			
			if (!isset($this->arguments["pres"]) || $this->arguments["pres"] == "") {
				Logger::Debug("ReportGenerator::isValid()", "Chart report doesn’t have a chart type");
				return false;	// chart type
			}
						
		} else if ($this->arguments["presentation_type"] == "table") {
			
			// Table variables
			//if (empty($this->arguments["xxx"])) return true;	// 
			
			
		} else {
			return false;
		}
			
		return true;
	}
	
	
	function generate() {

		if (is_null($this->SERVER)) return GENERATE_REPORT_ERROR_NOSERVER;
		
		$URL = $this->SERVER . ReportGenerator::$URL;
		
		// Parese arguments
		foreach ($this->arguments as $key => $value) {
			
			// Remove GUI specific things
			if ($key == "presentation_type") continue;
			if ($key == "debug") continue;
			
			
			if ($key == "threshold") {
				$URL .= urlencode($key) . "=" . ($value) . "&";
				continue;
			}
			$URL .= urlencode($key) . "=" . urlencode($value) . "&";
		}
		
		
		// Getting classifications from the DataType
		$classifications = $this->dataType->getClassifications();
		
		// Generate classification list
		$classificationParams = "";
		$anon = "";
		$int = 0;
		foreach ($classifications as $classification) {
			if ($classification->getId() == 0) continue;
			$int++;
			
			for ($i=$int; $i<$classification->getID(); $i++) {
				$int++;
				$classificationParams .= "x:";
			}
			
			$values = $classification->getSelectedClassificationValues();
			
			if (!$classification->isSelected()) {
				$classificationParams .= "x:";
				continue;
			}

			// Check if the classification needs to use anonymoused values
			if ($classification->isUseAnonymous() || isset($this->anonymousClassificationInternalIds[$classification->getId()])) {
				$anon .= $classification->getValue() . ",";
				Logger::Debug("ReportGenerator::generate()", "Setting classification '" . $classification->getValue() .  "' to be anonumous.");
			}
			
			// Adding classifications values to the URL
			if (isset($values) && is_array($values) && count($values) != 0) {
				Logger::Debug("ReportGenerator::generate()", "Classification '" . $classification->getValue() .  "' has selected values, adding them to URL.");
				$classificationParams .= "[";
				foreach ($values as $classificationValue) {
					$classValue = $classificationValue->getValue(false);
					$classValue = strcmp($classValue,"<empty>")== 0 ? " " : $classValue;
					$classificationParams .= str_replace(array('&', ','), array('&#x26;', '&#x2c;'), $classValue) . ",";
				}
				$classificationParams = substr($classificationParams, 0, -1);
				$classificationParams .= "]:";
			} else {
				$classificationParams .= "[]:";
			}

		}
		
		// Fill out the rest of the query string
		for ($i=$int; $i<11; $i++) {
			$classificationParams .= "x:";
		}
		
		//$return = array(); $return["STATUS"] = "ERROR"; $return["MESSAGE"] = $classificationParams; echo php2js($return); return;
		
		$URL .= "classvars=" . urlencode($classificationParams);
		
		if (!empty($anon)) {
			$URL .= "&anonymize=" . urlencode(substr($anon, 0, -1));
		}
		
		// Get locale
		if (isset($_SERVER["HTTP_ACCEPT_LANGUAGE"])) {
			if (substr_count($_SERVER["HTTP_ACCEPT_LANGUAGE"], ",") > 1) {
				list($locCode, $na, $na) = explode(",", $_SERVER["HTTP_ACCEPT_LANGUAGE"]);
				if (substr_count($locCode, "-") != 0) {
					list($locLanguage, $locCountry) = explode("-", $locCode);
				}
			}
		}

		// Set locale if we don't have it
		if (!isset($locLanguage) && !isset($locCountry)) {
			$locLanguage = "en";
			$locCountry = "US";
		}

		// Add the locale to the URL
		$URL .= "&loc=" . $locCountry . "," . $locLanguage . "," . ServerConfig::getWeekFormat();
		
		Logger::Debug("ReportGenerator::generate()", "URL : " . $URL);

		// Make sure that we can run this for a while
		set_time_limit(0);
		ini_set("default_socket_timeout", 600); // 10 min
		
		
		// Debug
		if (ServerConfig::getLogLevel() == 0) {
			$URL .= "&debug=on";
		}
		
		// Space fix
		$URL = str_replace(" " , "+" , $URL);
		
		// Grab response	
		//$response = file($URL);	// This uses GET, and want always work - Replacing with POST

		$pUrl = parse_url($URL); 

		$SEND = "POST " . $pUrl["scheme"] . "://" . $pUrl["host"] . (isset($pUrl["port"]) ? ":" . $pUrl["port"] : "") . $pUrl["path"] . " HTTP/1.0" . "\r\n" . 
                "Host: " . $pUrl["host"] . "\r\n" .
                "Accept: text/html, text/xml" . "\r\n" .
                "Accept-Language: en-us,en;q=0.5" . "\r\n" .
                "Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7" . "\r\n" .
                "Keep-Alive: 300" . "\r\n" .
                "Pragma: no-cache" . "\r\n" .
                "Cache-Control: no-cache" . "\r\n" .
                "Connection: keep-alive" . "\r\n" .
                "Content-Type: application/x-www-form-urlencoded; charset=UTF-8" . "\r\n" .
                "Content-Length: " .strlen($pUrl["query"]) ."\r\n\r\n" . $pUrl["query"];
       
		// Stop session
		session_write_close();		
		
		// Open connection
    	$open = fsockopen($pUrl["host"], (isset($pUrl["port"]) ? $pUrl["port"] : 80), $errno, $errstr, 100000000000);
    
	    // Send data
	    fputs($open, $SEND);
	    Logger::Debug("SEND", $SEND);
	    
	    // Variables
		$isHeader = true;
	    $response = "";
	    
		// Get content
	    while (!feof($open)) {
	    	$partData = fgets($open, 4096);
	    	if ($isHeader) {
			Logger::Debug("header", $partData);
				if (trim($partData) == "") $isHeader = false;
				continue;
	    	}
	    	
	    	if (trim($partData) == "") continue;	    	
			Logger::Debug("partData", $partData);
			$response .= $partData;
		}
		
		// Close connection
		fclose($open);
		Logger::Debug("response", $response);
		Logger::Debug("errno", $errno);
		Logger::Debug("errstr", $errstr);
		
		// Return
		$return = array();
		
		// Check for error messages
		if (substr_count($response, "WARNING") != 0) {
			$return["STATUS"] = "ERROR";
			$return["MESSAGE"] = $response;
			echo php2js($return);
			return;
		}
		
		// Get date time and id
		@list ($date, $time, $id) = @explode(" ", trim($response));
		
		// If we don't have an id, we probably have an error...
		if (!is_numeric($id)) {
			$return["STATUS"] = "ERROR";
			
			// Handle some of the known error messages
			
			// Date before 1990-01-01
			if (strstr($response, "is not a valid date")) {
				// Return only the error message
				//$return["MESSAGE"] = substr($response, strrpos($response, ":")+1);	// pass the information on..
				$return["MESSAGE"] = "Date before year 1990 is not supported.";
			}
			
			// default - return the whole error message
			else {
				$return["MESSAGE"] = $response;
			}
			
			echo php2js($return);
			return;
		}
		
		// Ok reponse
		$return["STATUS"] = "OK";
		$return["URL"] = ServerConfig::getClientUrl() . trim(file_get_contents(ServerConfig::getServerUrl() . "/cron_admin.pl?i=" . urlencode("+" . $id) . "&t=0&folder=" . urlencode($this->arguments["folder"]) . "&command=view")) . "";
		echo php2js($return);
		return;
				
	}
	
	
	
}

?>